#version 130
#extension GL_EXT_gpu_shader4 : enable

//##COMMON_DEFINITIONS_SHADER##

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int dissolveMode;

uniform vec4 viewport;
uniform float baseDistance;

in float pointSize;
in float colorCurveS;
in float colorCurveT;
in float filterAlpha;
in int paramBit;

out vec3 modelview_position;
out vec4 color;
out vec4 colorDiffuse;
out vec4 colorSpecular;
out vec2 center;
out float radius;
flat out int blend;
flat out int pseudoSphere;
flat out int boxPoint;
flat out int textureImageIndex; // valid 0-3 not use if 4;
//flat out int paramBitInt;

void main(void)
{
    modelview_position = vec3(gl_ModelViewMatrix * gl_Vertex);
    
    vec4 baseColor = texture(textureColorCurve, vec2(colorCurveS, colorCurveT));
    baseColor[3] *= filterAlpha;

    vec4 colorDiffuseBase = texture(textureColorCurve, vec2(colorCurveS, colorCurveT+0.3333));
    vec4 colorSpecularBase = texture(textureColorCurve, vec2(colorCurveS, colorCurveT+0.6666));
    
    float alpha0 = baseColor[3];

    if (dissolveMode == 1)
	if (baseColor[3] < 0.015)
	    alpha0 = 1;
    
    if (dissolveMode == 2)
	alpha0 = 1;
    
    color = baseColor;//vec4(baseColor[0] * alpha0, baseColor[1] * alpha0, baseColor[2] * alpha0, baseColor[3]);
    colorDiffuse = vec4(colorDiffuseBase[0] * alpha0, colorDiffuseBase[1] * alpha0, colorDiffuseBase[2] * alpha0, baseColor[3]);
    colorSpecular = vec4(colorSpecularBase[0] * alpha0, colorSpecularBase[1] * alpha0, colorSpecularBase[2] * alpha0, baseColor[3]);
    
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;//ftransform();

    /*
    float theta = atan(xydistance/gl_Position.w);
    float factor = 1.0 / cos(theta);
    */

    if (baseDistance != 0){
	gl_PointSize = pointSize / gl_Position.w * baseDistance /** factor */;
	radius = pointSize / gl_Position.w * 0.5 * baseDistance /** factor */;
    }
    else{
	gl_PointSize = pointSize /** factor*/;
	radius = pointSize * 0.5 /** factor*/;
    }
    
    blend = paramBit % _PARAMBIT_BLEND_MOD;
    pseudoSphere = (paramBit / _PARAMBIT_SPHERE) % 2;
    boxPoint = (paramBit / _PARAMBIT_BOXPOINT) % 2;

    if ((paramBit / _PARAMBIT_USE_TEXTURE) % 2 == 0)
	textureImageIndex = 4;
    else
	textureImageIndex = (paramBit / _PARAMBIT_TEXTUREINDEX) % _PARAMBIT_TEXTUREINDEX_MOD;
    
    center = vec2((gl_Position.x / gl_Position.w * viewport[2] + viewport[2])/2 + viewport[0],
		  (gl_Position.y / gl_Position.w * viewport[3] + viewport[3])/2 + viewport[1]);
}
