// PointBillboard.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_POINTS
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLE_STRIP
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=4

#version 150
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

layout (points) in;
//layout (triangles) in;
//layout (triangles_adjacency) in;
layout (triangle_strip, max_vertices = 4) out;

//##COMMON_DEFINITIONS_SHADER##

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int dissolveMode;

// ModelMatrix = viewInv * ModelView;
uniform mat4 baseInvModelMatrix;//
uniform mat4 modelInvBaseMatrix;//
uniform mat4 modelViewMatrix;
uniform mat4 modelViewProjectionMatrix;
uniform int twistMode;
uniform int ortho;
uniform float LODThreshold;
uniform float LODCritRadius;

//in vec3 modelview_positionIn[];
in float pointSizeIn[];
//in vec4 colorIn[];
//in vec4 colorDiffuseIn[];
//in vec4 colorSpecularIn[];

in float colorCurveSIn[];
in float colorCurveTIn[];
in float filterAlphaIn[];

out vec3 modelview_position;
out vec4 modelview_position4;
out vec4 color;
out vec4 colorDiffuse;
out vec4 colorSpecular;
out vec4 normal;
out vec4 tangent;
out vec4 binormal;
out vec2 gsTexCoord;

flat in int twistIDIn[];
flat in int paramBitIn[];

flat out int blend;
flat out int pseudoSphere;
flat out int textureImageIndex;

void main(void){

    int length = gl_PositionIn.length();
    //length = 1;

    float LOD2 = LODThreshold * LODThreshold;
    
    if (twistMode == 0){
	for (int i = 0; i < length; i++){

	    vec4 posIn = gl_PositionIn[i];
	    //float depth_z = (modelViewProjectionMatrix * posIn).z;
	    float depth_z = -(modelViewMatrix * posIn).z;

	    if (depth_z < 0) continue;
	    
	    float psize = pointSizeIn[i];

	    if (psize < LODCritRadius){
		if (depth_z > LODThreshold){
		    float downsize_remain = min(1, LOD2/depth_z/depth_z);
		    //float downsize_remain = 0.1;
	    
		    //float crit = fract(sin(dot(posIn.xy, vec2(1,1))) * 43758);
		    float crit = fract(dot(posIn.xy,vec2(1,1)) * 43758);
		    if (crit > downsize_remain) continue;
		
		    //float downsize_scale_factor = sqrt(1/downsize_remain);
		    float downsize_scale_factor = min(10, depth_z / LODThreshold);
		    psize *= downsize_scale_factor;
		}
	    }
	    
	    vec4 baseCoord = baseInvModelMatrix * posIn;
	    
	    vec4 baseColor = texture(textureColorCurve, vec2(colorCurveSIn[i], colorCurveTIn[i]));
	    baseColor[3] *= filterAlphaIn[i];
	    
	    vec4 colorDiffuseBase = texture(textureColorCurve, vec2(colorCurveSIn[i], colorCurveTIn[i]+0.3333));
	    vec4 colorSpecularBase = texture(textureColorCurve, vec2(colorCurveSIn[i], colorCurveTIn[i]+0.6666));
	    


	    float alpha0 = baseColor[3];

	    if (dissolveMode == 1)
		if (baseColor[3] < 0.015)
		    alpha0 = 1;
	    
	    if (dissolveMode == 2)
		alpha0 = 1;

	    color = baseColor;
	    colorDiffuse = vec4(colorDiffuseBase[0] * alpha0, colorDiffuseBase[1] * alpha0, colorDiffuseBase[2] * alpha0, baseColor[3]);
	    colorSpecular = vec4(colorSpecularBase[0] * alpha0, colorSpecularBase[1] * alpha0, colorSpecularBase[2] * alpha0, baseColor[3]);

	    float x = baseCoord[0];
	    float y = baseCoord[1];
	    float z = baseCoord[2];
	
	    float norm = sqrt(x*x+y*y+z*z);
	    if (norm == 0) continue;

	    x /= norm;
	    y /= norm;
	    z /= norm;
	
	    float sqxy = sqrt(x*x+y*y);
       
	    vec4 normalBase = vec4(x, y, z, 0);
	    vec4 tangentBase = -vec4(-y/sqxy, x/sqxy, 0, 0);
	    vec4 binormalBase = -vec4(-x*z/sqxy, -y*z/sqxy, sqxy, 0);

	    if (ortho == 1){
		normalBase   = vec4(0,0,-1, 0);
		tangentBase  = vec4(1,0,0, 0);
		binormalBase = vec4(0,1,0, 0);
	    }

	    normal = modelInvBaseMatrix * normalBase;
	    tangent  = modelInvBaseMatrix * tangentBase;
	    binormal = modelInvBaseMatrix * binormalBase;

	    vec4 offset[4] = vec4[]((-tangent-binormal), (+tangent-binormal),
				    (-tangent+binormal), (+tangent+binormal));
	    vec2 gs[4] = vec2[](vec2(1,0), vec2(0, 0), vec2(1,1), vec2(0,1));

	    normal = modelViewMatrix * normal;
	    tangent = modelViewMatrix * tangent;
	    binormal = modelViewMatrix * binormal;

	    int paramBit = paramBitIn[i];
	    blend = paramBit % _PARAMBIT_BLEND_MOD;
	    pseudoSphere = (paramBit / _PARAMBIT_SPHERE) % 2;
	    if ((paramBit / _PARAMBIT_USE_TEXTURE) % 2 == 0)
		textureImageIndex = 4;
	    else
		textureImageIndex = (paramBit / _PARAMBIT_TEXTUREINDEX) % _PARAMBIT_TEXTUREINDEX_MOD;

	    //modelview_position = modelview_positionIn[i];
	    modelview_position = vec3(modelViewMatrix * posIn);

	    if (color[3] > 0){

		gsTexCoord.st = gs[0];
		gl_Position = modelViewProjectionMatrix * (posIn + offset[0] * psize) ;
		modelview_position4 = modelViewMatrix * (posIn + offset[0] * psize) ;
		EmitVertex();
		
		gsTexCoord.st = gs[2];
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[2] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[2] * psize) ;
		EmitVertex();
		
		gsTexCoord.st = gs[1];
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[1] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[1] * psize) ;
		EmitVertex();

		/*
		gsTexCoord.st = gs[2];
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[2] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[2] * psize) ;
		EmitVertex();
		
		gsTexCoord.st = gs[1];
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[1] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[1] * psize) ;
		EmitVertex();
		*/
		
		gsTexCoord.st = gs[3];
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[3] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[3] * psize); 
		EmitVertex();

		EndPrimitive();
	    }
	}
    }
    else{
	float rad0 = -3.14159/4;
	float rad1 = 3.14159/4;
	float rad2 = 3.14159*3/4;
	float rad3 = 3.14159*5/4;

	float radius = sqrt(2)/2;
	for (int i = 0; i < length; i++){

	    vec4 posIn = gl_PositionIn[i];
	    //float depth_z = (modelViewProjectionMatrix * posIn).z;
	    float depth_z = -(modelViewProjection * posIn).z;
	    
	    if (depth_z < 0) return;
	    
	    float psize = pointSizeIn[i];

	    if (psize < LODCritRadius){
		float downsize_remain = min(1, LODThreshold/depth_z/depth_z);
		//float downsize_remain = 0.1;
	    
		float crit = fract(sin(dot(posIn.xy, vec2(1,1))) * 43758);
		if (crit > downsize_remain) return;
		
		float downsize_scale_factor = sqrt(1/downsize_remain);
		psize *= downsize_scale_factor;
	    }
	
	    vec4 baseCoord = baseInvModelMatrix * gl_PositionIn[i];

	    vec4 baseColor = texture(textureColorCurve, vec2(colorCurveSIn[i], colorCurveTIn[i]));
	    baseColor[3] *= filterAlphaIn[i];
	    
	    vec4 colorDiffuseBase = texture(textureColorCurve, vec2(colorCurveSIn[i], colorCurveTIn[i]+0.3333));
	    vec4 colorSpecularBase = texture(textureColorCurve, vec2(colorCurveSIn[i], colorCurveTIn[i]+0.6666));
	    
	    float alpha0 = baseColor[3];

	    if (dissolveMode == 1)
		if (baseColor[3] < 0.015)
		    alpha0 = 1;
	    
	    if (dissolveMode == 2)
		alpha0 = 1;

	    color = baseColor;
	    colorDiffuse = vec4(colorDiffuseBase[0] * alpha0, colorDiffuseBase[1] * alpha0, colorDiffuseBase[2] * alpha0, baseColor[3]);
	    colorSpecular = vec4(colorSpecularBase[0] * alpha0, colorSpecularBase[1] * alpha0, colorSpecularBase[2] * alpha0, baseColor[3]);
	    
	    float x = baseCoord[0];
	    float y = baseCoord[1];
	    float z = baseCoord[2];
	
	    float norm = sqrt(x*x+y*y+z*z);
	    if (norm == 0) continue;

	    x /= norm;
	    y /= norm;
	    z /= norm;
	
	    float sqxy = sqrt(x*x+y*y);
       
	    vec4 normalBase = vec4(x, y, z, 0);
	    vec4 tangentBase = -vec4(-y/sqxy, x/sqxy, 0, 0);
	    vec4 binormalBase = -vec4(-x*z/sqxy, -y*z/sqxy, sqxy, 0);

	    normal = modelInvBaseMatrix * normalBase;
	    tangent  = modelInvBaseMatrix * tangentBase;
	    binormal = modelInvBaseMatrix * binormalBase;

	    vec4 offset[4] = vec4[]((-tangent-binormal), (+tangent-binormal),
				    (-tangent+binormal), (+tangent+binormal));

	    normal   = modelViewMatrix * normal;
	    tangent  = modelViewMatrix * tangent;
	    binormal = modelViewMatrix * binormal;

	    int paramBit = paramBitIn[i];
	    blend = paramBit % _PARAMBIT_BLEND_MOD;
	    pseudoSphere = (paramBit / _PARAMBIT_SPHERE) % 2;
	    if ((paramBit / _PARAMBIT_USE_TEXTURE) % 2 == 0)
		textureImageIndex = 4;
	    else
		textureImageIndex = (paramBit / _PARAMBIT_TEXTUREINDEX) % _PARAMBIT_TEXTUREINDEX_MOD;
	
	    
	    //modelview_position = modelview_positionIn[i];
	    modelview_position = vec3(modelViewMatrix * posIn);

	    float twist = float(twistIDIn[i] % 1000);

	    if (color[3] > 0){
		gsTexCoord.st = vec2(radius*cos(rad0+twist)+0.5, radius*sin(rad0+twist)+0.5);
		gl_Position = modelViewProjectionMatrix * (posIn + offset[0] * psize) ;
		modelview_position4 = modelViewMatrix * (posIn + offset[0] * psize) ;
		EmitVertex();
	    
		gsTexCoord = vec2(radius*cos(rad1+twist)+0.5, radius*sin(rad1+twist)+0.5);
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[2] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[2] * psize) ;
		EmitVertex();
	    
		gsTexCoord = vec2(radius*cos(rad3+twist)+0.5, radius*sin(rad3+twist)+0.5);
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[1] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[1] * psize) ;
		EmitVertex();

		/*
		gsTexCoord = vec2(radius*cos(rad1+twist)+0.5, radius*sin(rad1+twist)+0.5);
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[2] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[2] * psize) ;
		EmitVertex();
	    
		gsTexCoord = vec2(radius*cos(rad3+twist)+0.5, radius*sin(rad3+twist)+0.5);
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[1] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[1] * psize) ;
		EmitVertex();
		*/
	    
		gsTexCoord = vec2(radius*cos(rad2+twist)+0.5, radius*sin(rad2+twist)+0.5);
		gl_Position = modelViewProjectionMatrix *  (posIn + offset[3] * psize);
		modelview_position4 = modelViewMatrix * (posIn + offset[1] * psize) ;
		EmitVertex();

		EndPrimitive();
	    }
	}
    }
}
