//#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage;
uniform sampler2D textureImage1;

uniform int quality;
uniform float projMatrix10;
uniform float projMatrix14;
uniform float factor;
uniform float radius;
uniform vec3 baseColor;
uniform int type;

#define MAXSAMPLES 256

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main (void)
{
    vec2 texCoord0 = gl_TexCoord[0].st;

    vec4 texColorOrg = texture2D(textureImage, texCoord0);
	    
    texColorOrg[0] += 0.0001;
    texColorOrg[1] += 0.0001;
    texColorOrg[2] += 0.0001;

    vec2 FragCoord = gl_FragCoord.xy;
    float OneMinusMatrix10 = 1.0-projMatrix10;

    int samples = quality * 4;//MAXSAMPLES;
    if (samples > MAXSAMPLES) samples = MAXSAMPLES;
    int count = 0;

    vec2 co3 = FragCoord * 7;
    
    for (int jy = -2; jy <=2; jy++){
	for (int jx = -2; jx <=2; jx++){
	    vec2 texCoord = texCoord0 + vec2(float(jx)/2048, float(jy)/2048);
	    
	    vec4 texColor1Base = texture2D(textureImage1, texCoord);
	    
	    float z_eyeBase = - projMatrix14 / (texColor1Base[0] * -2.0 + OneMinusMatrix10);
	    float sampleRadius = radius / z_eyeBase;
	    
	    vec2 co = FragCoord * 3 * (jx+3);
	    vec2 co2 = FragCoord * 5 * (jy+3);
	    
	    float offset = 0;//mod(z_eyeBase, 1);
	    
	    for (int i = 0; i < samples; i++){
		float r = sampleRadius * rand(co+vec2(i,offset+i)) ;
		float t = 6.2831853 * rand(co2+vec2(i*2+jx,offset*2-i*3)) ;
		float cp = 2.0 * rand(co3+vec2(i+jy,2+i*3)) - 1.0;
		float sp = sqrt(1.0 - cp * cp);
		float ct = cos(t), st = sin(t);
		
		float dx = r * sp * ct;
		float dy = r * sp * st;
		float dz = r * cp;
		
		vec4 texColor1 = texture2D(textureImage1, texCoord+vec2(dx, dy));
		float z_eye = - projMatrix14 / (texColor1[0] * -2.0 + OneMinusMatrix10);
		if (z_eye > z_eyeBase + dz) count++;
	    }
	    
	}
    }

    // a: enlit factor
    //float a = float(count) / samples / 25 * 3;
    float a = float(count) / samples / 25 * 2;//
    
    if (type == 0){
	a = (tanh(5*(a-0.5)/2)+1)/2;
	a = (a-1)*factor + 1;
	if (a < 0) a = 0;
    }
    if (type == 1){
	a = (tanh(5*(a/2-0.5)/2)+1)/2;
	a = a*factor + 1;
	if (a < 1) a = 1;
    }
    if (type == 2){
	a = (tanh(5*(a/2-0.5))+1);
	a = (a-1)*factor*0.5 + 1 ;
    }

    gl_FragColor[0] = texColorOrg[0] * (1.0 - (1.0-baseColor[0]) * (1.0 - a));
    gl_FragColor[1] = texColorOrg[1] * (1.0 - (1.0-baseColor[1]) * (1.0 - a));
    gl_FragColor[2] = texColorOrg[2] * (1.0 - (1.0-baseColor[2]) * (1.0 - a));
    gl_FragColor[3] = texColorOrg[3];
	
    return;
}
